@@ -409,7 +409,7 @@ class Texture(_vtk.vtkTexture, DataObject):
         elif image.ndim == 2:
             n_components = 1
 
-        grid = pyvista.UniformGrid(dims=(image.shape[1], image.shape[0], 1))
+        grid = pyvista.UniformGrid(dimensions=(image.shape[1], image.shape[0], 1))
         grid.point_data['Image'] = np.flip(image.swapaxes(0, 1), axis=1).reshape(
             (-1, n_components), order='F'
         )
