@@ -3,7 +3,6 @@
 from array import array
 import warnings
 
-from IPython import display
 import numpy as np
 
 # not to be imported at the init level
@@ -115,7 +114,7 @@ def ipygany_block_from_actor(actor):
     mapper = actor.GetMapper()
     if mapper is None:
         return
-    dataset = mapper.GetInputAsDataSet()
+    dataset = pv.wrap(mapper.GetInputAsDataSet())
 
     prop = actor.GetProperty()
     rep_type = prop.GetRepresentationAsString()
@@ -161,7 +160,7 @@ def ipygany_camera_from_plotter(plotter):
     return {'position': position.tolist(), 'target': target, 'up': up}
 
 
-def show_ipygany(plotter, return_viewer, height=None, width=None):
+def show_ipygany(plotter, height=None, width=None):
     """Show an ipygany scene."""
     # convert each mesh in the plotter to an ipygany scene
     actors = plotter.renderer._actors
@@ -188,13 +187,10 @@ def show_ipygany(plotter, return_viewer, height=None, width=None):
                 colored_mesh = mesh
                 break
 
-    # Simply return the scene
-    if return_viewer:
-        return scene
-
     if cbar is not None:
         # Colormap choice widget
-        colormap_dd = Dropdown(options=colormaps, description='Colormap:')
+        options = list(colormaps.items())
+        colormap_dd = Dropdown(options=options, description='Colormap:')
         jslink((colored_mesh, 'colormap'), (colormap_dd, 'index'))
 
         # sensible colorbar maximum width, or else it looks bad when
@@ -216,4 +212,4 @@ def show_ipygany(plotter, return_viewer, height=None, width=None):
         legend = VBox((title, colormap_dd, cbar))
         scene = AppLayout(center=scene, footer=legend, pane_heights=[0, 0, '150px'])
 
-    display.display_html(scene)
+    return scene
