@@ -1,20 +1,20 @@
 """Plotting routines."""
 
 from pyvista import MAX_N_COLOR_BARS
-from .charts import Chart2D, ChartMPL, ChartBox, ChartPie
+from .charts import Chart, Chart2D, ChartMPL, ChartBox, ChartPie
 from .colors import (
     Color,
-    color_like,
+    ColorLike,
     color_char_to_word,
     get_cmap_safe,
-    hex_to_rgb,
     hexcolors,
-    string_to_rgb,
     PARAVIEW_BACKGROUND,
 )
+from .composite_mapper import CompositeAttributes, BlockAttributes, CompositePolyDataMapper
 from .export_vtkjs import export_plotter_vtkjs, get_vtkjs_url
-from .helpers import plot, plot_arrows, plot_compare_four, plot_itk
+from .helpers import plot, plot_arrows, plot_compare_four
 from .plotting import BasePlotter, Plotter, close_all
+from ._property import Property
 from .renderer import CameraPosition, Renderer, scale_point
 from .tools import (
     create_axes_marker,
@@ -22,13 +22,17 @@ from .tools import (
     opacity_transfer_function,
     FONTS,
     system_supports_plotting,
-    parse_color,
     parse_font_family,
 )
 from .widgets import WidgetHelper
 from .lights import Light
 from .camera import Camera
 from .axes import Axes
+from .axes_actor import AxesActor
+from .actor import Actor
+from .actor_properties import ActorProperties
+from .mapper import DataSetMapper
+from .lookup_table import LookupTable
 
 
 class QtDeprecationError(Exception):
