@@ -1,6 +1,7 @@
 """Fine-grained control of reading data files."""
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
+import enum
 from functools import wraps
 import os
 import pathlib
@@ -68,6 +69,10 @@ def get_reader(filename, force_ext=None):
     +----------------+---------------------------------------------+
     | ``.mhd``       | :class:`pyvista.MetaImageReader`            |
     +----------------+---------------------------------------------+
+    | ``.nii``       | :class:`pyvista.NIFTIReader`                |
+    +----------------+---------------------------------------------+
+    | ``.nii.gz``    | :class:`pyvista.NIFTIReader`                |
+    +----------------+---------------------------------------------+
     | ``.nhdr``      | :class:`pyvista.NRRDReader`                 |
     +----------------+---------------------------------------------+
     | ``.nrrd``      | :class:`pyvista.NRRDReader`                 |
@@ -166,6 +171,51 @@ def get_reader(filename, force_ext=None):
     return Reader(filename)
 
 
+class BaseVTKReader(ABC):
+    """Simulate a VTK reader."""
+
+    def __init__(self):
+        self._data_object = None
+        self._observers: List[Union[int, Callable]] = []
+
+    def SetFileName(self, filename):
+        """Set file name."""
+        self._filename = filename
+
+    def UpdateInformation(self):
+        """Update Information from file."""
+        pass
+
+    def AddObserver(self, event_type, callback):
+        """Add Observer that can be triggered during Update."""
+        self._observers.append([event_type, callback])
+
+    def RemoveObservers(self, *args):
+        """Remove Observer."""
+        self._observers = []
+
+    def GetProgress(self):
+        """GetProgress."""
+        return 0.0 if self._data_object is None else 1.0
+
+    def UpdateObservers(self, event_type):
+        """Call matching observer."""
+        for event_type_allowed, observer in self._observers:
+            if event_type_allowed == event_type:
+                observer(self, event_type)
+
+    @abstractmethod
+    def Update(self):
+        """Update Reader from file and store data internally.
+
+        Set self._data_object.
+        """
+
+    def GetOutputDataObject(self, *args):
+        """Return stored data."""
+        return self._data_object
+
+
 @abstract_class
 class BaseReader:
     """The Base Reader class.
@@ -304,8 +354,11 @@ class BaseReader:
         _update_alg(self.reader, progress_bar=self._progress_bar, message=self._progress_msg)
         data = wrap(self.reader.GetOutputDataObject(0))
         if data is None:  # pragma: no cover
-            raise RuntimeError("Failed to read file.")
+            raise RuntimeError("File reader failed to read and/or produced no output.")
         data._post_file_load_processing()
+
+        # check for any pyvista metadata
+        data._restore_metadata()
         return data
 
     def _update_information(self):
@@ -741,7 +794,7 @@ class EnSightReader(BaseReader, PointCellDataSelection, TimeReader):
     def set_active_time_value(self, time_value):  # noqa: D102
         if time_value not in self.time_values:
             raise ValueError(
-                f"Not a valid time {time_value} from available time values: {self.reader_time_values}"
+                f"Not a valid time {time_value} from available time values: {self.time_values}"
             )
         self.reader.SetTimeValue(time_value)
 
@@ -964,6 +1017,7 @@ class OpenFOAMReader(BaseReader, PointCellDataSelection, TimeReader):
         ----------
         name : str
             Which patch array to report status.
+
         Returns
         -------
         bool
@@ -1245,11 +1299,81 @@ class Plot3DMetaReader(BaseReader):
     _class_reader = staticmethod(_vtk.lazy_vtkPlot3DMetaReader)
 
 
+class Plot3DFunctionEnum(enum.IntEnum):
+    """An enumeration for the functions used in :class:`MultiBlockPlot3DReader`."""
+
+    DENSITY = 100
+    PRESSURE = 110
+    PRESSURE_COEFFICIENT = 111
+    MACH = 112
+    SPEED_OF_SOUND = 113
+    TEMPERATURE = 120
+    ENTHALPY = 130
+    INTERNAL_ENERGY = 140
+    KINETIC_ENERGY = 144
+    VELOCITY_MAGNITUDE = 153
+    STAGNATION_ENERGY = 163
+    ENTROPY = 170
+    SWIRL = 184
+    VELOCITY = 200
+    VORTICITY = 201
+    MOMENTUM = 202
+    PRESSURE_GRADIENT = 210
+    STRAIN_RATE = 212
+    VORTICITY_MAGNITUDE = 211
+
+
 class MultiBlockPlot3DReader(BaseReader):
-    """MultiBlock Plot3D Reader."""
+    """MultiBlock Plot3D Reader.
+
+    The methods :meth:`add_function()` and :meth:`remove_function()` accept values from
+    :class:`Plot3DFunctionEnum`. For convenience, the values of that enumeration are available as class variables,
+    as shown below.
+
+        - ``MultiBlockPlot3DReader.DENSITY = Plot3DFunctionEnum.DENSITY``
+        - ``MultiBlockPlot3DReader.PRESSURE = Plot3DFunctionEnum.PRESSURE``
+        - ``MultiBlockPlot3DReader.PRESSURE_COEFFICIENT = Plot3DFunctionEnum.PRESSURE_COEFFICIENT``
+        - ``MultiBlockPlot3DReader.MACH = Plot3DFunctionEnum.MACH``
+        - ``MultiBlockPlot3DReader.SPEED_OF_SOUND = Plot3DFunctionEnum.SPEED_OF_SOUND``
+        - ``MultiBlockPlot3DReader.TEMPERATURE = Plot3DFunctionEnum.TEMPERATURE``
+        - ``MultiBlockPlot3DReader.ENTHALPY = Plot3DFunctionEnum.ENTHALPY``
+        - ``MultiBlockPlot3DReader.INTERNAL_ENERGY = Plot3DFunctionEnum.INTERNAL_ENERGY``
+        - ``MultiBlockPlot3DReader.KINETIC_ENERGY = Plot3DFunctionEnum.KINETIC_ENERGY``
+        - ``MultiBlockPlot3DReader.VELOCITY_MAGNITUDE = Plot3DFunctionEnum.VELOCITY_MAGNITUDE``
+        - ``MultiBlockPlot3DReader.STAGNATION_ENERGY = Plot3DFunctionEnum.STAGNATION_ENERGY``
+        - ``MultiBlockPlot3DReader.ENTROPY = Plot3DFunctionEnum.ENTROPY``
+        - ``MultiBlockPlot3DReader.SWIRL = Plot3DFunctionEnum.SWIRL``
+        - ``MultiBlockPlot3DReader.VELOCITY = Plot3DFunctionEnum.VELOCITY``
+        - ``MultiBlockPlot3DReader.VORTICITY = Plot3DFunctionEnum.VORTICITY``
+        - ``MultiBlockPlot3DReader.MOMENTUM = Plot3DFunctionEnum.MOMENTUM``
+        - ``MultiBlockPlot3DReader.PRESSURE_GRADIENT = Plot3DFunctionEnum.PRESSURE_GRADIENT``
+        - ``MultiBlockPlot3DReader.STRAIN_RATE = Plot3DFunctionEnum.STRAIN_RATE``
+        - ``MultiBlockPlot3DReader.VORTICITY_MAGNITUDE = Plot3DFunctionEnum.VORTICITY_MAGNITUDE``
+    """
 
     _class_reader = staticmethod(_vtk.lazy_vtkMultiBlockPLOT3DReader)
 
+    # pull in function name enum values as class constants
+    DENSITY = Plot3DFunctionEnum.DENSITY
+    PRESSURE = Plot3DFunctionEnum.PRESSURE
+    PRESSURE_COEFFICIENT = Plot3DFunctionEnum.PRESSURE_COEFFICIENT
+    MACH = Plot3DFunctionEnum.MACH
+    SPEED_OF_SOUND = Plot3DFunctionEnum.SPEED_OF_SOUND
+    TEMPERATURE = Plot3DFunctionEnum.TEMPERATURE
+    ENTHALPY = Plot3DFunctionEnum.ENTHALPY
+    INTERNAL_ENERGY = Plot3DFunctionEnum.INTERNAL_ENERGY
+    KINETIC_ENERGY = Plot3DFunctionEnum.KINETIC_ENERGY
+    VELOCITY_MAGNITUDE = Plot3DFunctionEnum.VELOCITY_MAGNITUDE
+    STAGNATION_ENERGY = Plot3DFunctionEnum.STAGNATION_ENERGY
+    ENTROPY = Plot3DFunctionEnum.ENTROPY
+    SWIRL = Plot3DFunctionEnum.SWIRL
+    VELOCITY = Plot3DFunctionEnum.VELOCITY
+    VORTICITY = Plot3DFunctionEnum.VORTICITY
+    MOMENTUM = Plot3DFunctionEnum.MOMENTUM
+    PRESSURE_GRADIENT = Plot3DFunctionEnum.PRESSURE_GRADIENT
+    STRAIN_RATE = Plot3DFunctionEnum.STRAIN_RATE
+    VORTICITY_MAGNITUDE = Plot3DFunctionEnum.VORTICITY_MAGNITUDE
+
     def _set_defaults(self):
         self.auto_detect_format = True
 
@@ -1292,6 +1416,85 @@ class MultiBlockPlot3DReader(BaseReader):
     def auto_detect_format(self, value):
         self.reader.SetAutoDetectFormat(value)
 
+    def add_function(self, value: Union[int, Plot3DFunctionEnum]):
+        """Specify additional functions to compute.
+
+        The available functions are enumerated in :class:`Plot3DFunctionEnum`. The members of this enumeration are most
+        easily accessed by their aliases as class variables.
+
+        Multiple functions may be requested by calling this method multiple times.
+
+        Parameters
+        ----------
+        value : int or Plot3DFunctionEnum
+            The function to add.
+
+        Examples
+        --------
+        >>> import pyvista
+        >>> from pyvista import examples
+        >>> filename  = examples.download_file('multi-bin.xyz')
+        >>> reader = pyvista.reader.MultiBlockPlot3DReader(filename)
+        >>> reader.add_function(112)  # add a function by its integer value
+        >>> reader.add_function(reader.PRESSURE_COEFFICIENT)  # add a function by enumeration via class variable alias
+
+        """
+        if isinstance(value, enum.Enum):
+            value = value.value
+        self.reader.AddFunction(value)
+
+    def remove_function(self, value: Union[int, Plot3DFunctionEnum]):
+        """Remove one function from list of functions to compute.
+
+        For details on the types of accepted values, see :meth:``add_function``.
+
+        Parameters
+        ----------
+        value : int or Plot3DFunctionEnum
+            The function to remove.
+        """
+        if isinstance(value, enum.Enum):
+            value = value.value
+        self.reader.RemoveFunction(value)
+
+    def remove_all_functions(self):
+        """Remove all functions from list of functions to compute."""
+        self.reader.RemoveAllFunctions()
+
+    @property
+    def preserve_intermediate_functions(self):
+        """When ``True`` (default), intermediate computed quantities will be preserved.
+
+        For example, if ``VelocityMagnitude`` is enabled, but not ``Velocity``, the reader still needs to compute
+        ``Velocity``. If `preserve_intermediate_functions` is ``False``, then the output will not have ``Velocity``
+        array, only the requested ``VelocityMagnitude``.
+
+        This is useful to avoid using up memory for arrays that are not relevant for the analysis.
+        """
+        return self.reader.GetPreserveIntermediateFunctions()
+
+    @preserve_intermediate_functions.setter
+    def preserve_intermediate_functions(self, val):
+        self.reader.SetPreserveIntermediateFunctions(val)
+
+    @property
+    def gamma(self):
+        """Ratio of specific heats."""
+        return self.reader.GetGamma()
+
+    @gamma.setter
+    def gamma(self, val):
+        self.reader.SetGamma(val)
+
+    @property
+    def r_gas_constant(self):
+        """Gas constant."""
+        return self.reader.GetR()
+
+    @r_gas_constant.setter
+    def r_gas_constant(self, val):
+        self.reader.SetR(val)
+
 
 class CGNSReader(BaseReader, PointCellDataSelection):
     """CGNS Reader for .cgns files.
@@ -1619,6 +1822,59 @@ class PVDDataSet:
     group: str
 
 
+class _PVDReader(BaseVTKReader):
+    """Simulate a VTK reader for GIF files."""
+
+    def __init__(self):
+        super().__init__()
+        self._directory = None
+        self._datasets = None
+        self._active_datasets = None
+        self._time_values = None
+
+    def SetFileName(self, filename):
+        """Set filename and update reader."""
+        self._filename = filename
+        self._directory = os.path.join(os.path.dirname(filename))
+
+    def UpdateInformation(self):
+        """Parse PVD file."""
+        if self._filename is None:
+            raise ValueError("Filename must be set")
+        tree = ElementTree.parse(self._filename)
+        root = tree.getroot()
+        dataset_elements = root[0].findall("DataSet")
+        datasets = []
+        for element in dataset_elements:
+            element_attrib = element.attrib
+            datasets.append(
+                PVDDataSet(
+                    float(element_attrib.get('timestep', 0)),
+                    int(element_attrib.get('part', 0)),
+                    element_attrib['file'],
+                    element_attrib.get('group'),
+                )
+            )
+        self._datasets = sorted(datasets)
+        self._time_values = sorted({dataset.time for dataset in self._datasets})
+
+        self._SetActiveTime(self._time_values[0])
+
+    def Update(self):
+        """Read data and store it."""
+        self._data_object = pyvista.MultiBlock([reader.read() for reader in self._active_readers])
+
+    def _SetActiveTime(self, time_value):
+        """Set active time."""
+        self._active_datasets = [
+            dataset for dataset in self._datasets if dataset.time == time_value
+        ]
+        self._active_readers = [
+            get_reader(os.path.join(self._directory, dataset.path))
+            for dataset in self._active_datasets
+        ]
+
+
 # skip pydocstyle D102 check since docstring is taken from TimeReader
 class PVDReader(BaseReader, TimeReader):
     """PVD Reader for .pvd files.
@@ -1641,26 +1897,7 @@ class PVDReader(BaseReader, TimeReader):
 
     """
 
-    def __init__(self, filename):
-        """Initialize PVD file reader."""
-        self._reader = None
-        self.__directory = None
-        self._datasets = []
-        self._active_datasets = []
-        self._active_readers = []
-        self._time_values = []
-
-        self._set_filename(filename)
-
-    @property
-    def reader(self):
-        """Return the PVDReader.
-
-        .. note::
-            This Reader does not have an underlying vtk Reader.
-
-        """
-        return self
+    _class_reader = _PVDReader
 
     @property
     def active_readers(self):
@@ -1671,7 +1908,7 @@ class PVDReader(BaseReader, TimeReader):
         list[pyvista.BaseReader]
 
         """
-        return self._active_readers
+        return self.reader._active_readers
 
     @property
     def datasets(self):
@@ -1682,7 +1919,7 @@ class PVDReader(BaseReader, TimeReader):
         list[pyvista.PVDDataSet]
 
         """
-        return self._datasets
+        return self.reader._datasets
 
     @property
     def active_datasets(self):
@@ -1693,78 +1930,26 @@ class PVDReader(BaseReader, TimeReader):
         list[pyvista.PVDDataSet]
 
         """
-        return self._active_datasets
-
-    def _set_filename(self, filename):
-        """Set filename and update reader."""
-        self._filename = filename
-        self.__directory = os.path.join(os.path.dirname(filename))
-        self._datasets = None
-        self._active_datasets = None
-        self._update_information()
-
-    def read(self):
-        """Read data from PVD timepoint.
-
-        Overrides :func:`pyvista.BaseReader.read`.
-
-        Returns
-        -------
-        pyvista.MultiBlock
-
-        """
-        return pyvista.MultiBlock([reader.read() for reader in self.active_readers])
-
-    def _update_information(self):
-        """If dataset information is unavailable, parse file."""
-        if self.datasets is None:
-            self._parse_file()
-
-    def _parse_file(self):
-        """Parse PVD file."""
-        if self.path is None:
-            raise ValueError("Filename must be set")
-        tree = ElementTree.parse(self.path)
-        root = tree.getroot()
-        dataset_elements = root[0].findall("DataSet")
-        datasets = []
-        for element in dataset_elements:
-            element_attrib = element.attrib
-            datasets.append(
-                PVDDataSet(
-                    float(element_attrib.get('timestep', 0)),
-                    int(element_attrib['part']),
-                    element_attrib['file'],
-                    element_attrib.get('group'),
-                )
-            )
-        self._datasets = sorted(datasets)
-        self._time_values = sorted(list(set([dataset.time for dataset in self.datasets])))
-
-        self.set_active_time_value(self.time_values[0])
+        return self.reader._active_datasets
 
     @property
     def time_values(self):  # noqa: D102
-        return self._time_values
+        return self.reader._time_values
 
     @property
     def number_time_points(self):  # noqa: D102
-        return len(self.time_values)
+        return len(self.reader._time_values)
 
     def time_point_value(self, time_point):  # noqa: D102
-        return self.time_values[time_point]
+        return self.reader._time_values[time_point]
 
     @property
     def active_time_value(self):  # noqa: D102
         # all active datasets have the same time
-        return self.active_datasets[0].time
+        return self.reader._active_datasets[0].time
 
     def set_active_time_value(self, time_value):  # noqa: D102
-        self._active_datasets = [dataset for dataset in self.datasets if dataset.time == time_value]
-        self._active_readers = [
-            get_reader(os.path.join(self.__directory, dataset.path))
-            for dataset in self.active_datasets
-        ]
+        self.reader._SetActiveTime(time_value)
 
     def set_active_time_point(self, time_point):  # noqa: D102
         self.set_active_time_value(self.time_values[time_point])
@@ -1843,7 +2028,7 @@ class JPEGReader(BaseReader):
     --------
     >>> import pyvista
     >>> from pyvista import examples
-    >>> filename = examples.download_mars_jpg()
+    >>> filename = examples.planets.download_mars_surface(load=False)
     >>> filename.split("/")[-1]  # omit the path
     'mars.jpg'
     >>> reader = pyvista.get_reader(filename)
@@ -1874,6 +2059,25 @@ class MetaImageReader(BaseReader):
     _class_reader = _vtk.vtkMetaImageReader
 
 
+class NIFTIReader(BaseReader):
+    """NIFTI Reader for .nii and .nii.gz files.
+
+    Examples
+    --------
+    >>> import pyvista
+    >>> from pyvista import examples
+    >>> filename = examples.download_brain_atlas_with_sides(load=False)
+    >>> filename.split("/")[-1]  # omit the path
+    'avg152T1_RL_nifti.nii.gz'
+    >>> reader = pyvista.get_reader(filename)
+    >>> mesh = reader.read()
+    >>> mesh.plot()
+
+    """
+
+    _class_reader = _vtk.vtkNIFTIImageReader
+
+
 class NRRDReader(BaseReader):
     """NRRDReader for .nrrd and .nhdr files.
 
@@ -2072,44 +2276,23 @@ class SegYReader(BaseReader):
     _class_reader = staticmethod(_vtk.lazy_vtkSegYReader)
 
 
-class _GIFReader:
+class _GIFReader(BaseVTKReader):
     """Simulate a VTK reader for GIF files."""
 
-    _data_object = None
-    _observers: List[Union[int, Callable]] = []
-    _n_frames = 0
-    _current_frame = 0
-
-    def SetFileName(self, filename):
-        """Needed for VTK-like compatibility with BaseReader."""
-        self._filename = filename
-
-    def UpdateInformation(self):
-        """Needed for VTK-like compatibility with BaseReader."""
-        pass
-
-    def AddObserver(self, event_type, callback):
-        """Needed for VTK-like compatibility with BaseReader."""
-        self._observers.append([event_type, callback])
-
-    def RemoveObservers(self, *args):
-        """Needed for VTK-like compatibility with BaseReader."""
-        self._observers = []
+    def __init__(self):
+        super().__init__()
+        self._n_frames = 0
+        self._current_frame = 0
 
     def GetProgress(self):
         return self._current_frame / self._n_frames
 
-    def UpdateObservers(self, event_type):
-        for event_type_allowed, observer in self._observers:
-            if event_type_allowed == event_type:
-                observer(self, event_type)
-
     def Update(self):
         """Read the GIF and store internally to `_data_object`."""
         from PIL import Image, ImageSequence
 
         img = Image.open(self._filename)
-        self._data_object = pyvista.UniformGrid(dims=(img.size[0], img.size[1], 1))
+        self._data_object = pyvista.UniformGrid(dimensions=(img.size[0], img.size[1], 1))
 
         # load each frame to the grid (RGB since gifs do not support transparency
         self._n_frames = img.n_frames
@@ -2122,10 +2305,6 @@ class _GIFReader:
         if 'frame0' in self._data_object.point_data:
             self._data_object.point_data.active_scalars_name = 'frame0'
 
-    def GetOutputDataObject(self, *args):
-        """Needed for VTK-like compatibility with BaseReader."""
-        return self._data_object
-
 
 class GIFReader(BaseReader):
     """GIFReader for .gif files.
@@ -2174,6 +2353,8 @@ CLASS_READERS = {
     '.hdr': HDRReader,
     '.mha': MetaImageReader,
     '.mhd': MetaImageReader,
+    '.nii': NIFTIReader,
+    '.nii.gz': NIFTIReader,
     '.nhdr': NRRDReader,
     '.nrrd': NRRDReader,
     '.obj': OBJReader,
