@@ -38,14 +38,39 @@ def wrap_image_array(arr):
     return wrap_img
 
 
-def image_from_window(ren_win, as_vtk=False, ignore_alpha=False):
+def run_image_filter(imfilter: _vtk.vtkWindowToImageFilter):
+    """Run a ``vtkWindowToImageFilter`` and get output as array."""
+    # Update filter and grab pixels
+    imfilter.Modified()
+    imfilter.Update()
+    image = pyvista.wrap(imfilter.GetOutput())
+    img_size = image.dimensions
+    img_array = pyvista.utilities.point_array(image, 'ImageScalars')
+    # Reshape and write
+    tgt_size = (img_size[1], img_size[0], -1)
+    return img_array.reshape(tgt_size)[::-1]
+
+
+def image_from_window(render_window, as_vtk=False, ignore_alpha=False, scale=1):
     """Extract the image from the render window as an array."""
-    width, height = ren_win.GetSize()
-    arr = _vtk.vtkUnsignedCharArray()
-    ren_win.GetRGBACharPixelData(0, 0, width - 1, height - 1, 0, arr)
-    data = _vtk.vtk_to_numpy(arr).reshape(height, width, -1)[::-1]
+    off = not render_window.GetInteractor().GetEnableRender()
+    if off:
+        render_window.GetInteractor().EnableRenderOn()
+    imfilter = _vtk.vtkWindowToImageFilter()
+    imfilter.SetInput(render_window)
+    imfilter.SetScale(scale)
+    imfilter.FixBoundaryOn()
+    imfilter.ReadFrontBufferOff()
+    imfilter.ShouldRerenderOff()
     if ignore_alpha:
-        data = data[:, :, :-1]
+        imfilter.SetInputBufferTypeToRGB()
+    else:
+        imfilter.SetInputBufferTypeToRGBA()
+    imfilter.ReadFrontBufferOn()
+    data = run_image_filter(imfilter)
+    if off:
+        # Critical for Trame and other offscreen tools
+        render_window.GetInteractor().EnableRenderOff()
     if as_vtk:
         return wrap_image_array(data)
     return data
@@ -64,12 +89,12 @@ def compare_images(im1, im2, threshold=1, use_vtk=True):
         Render window, numpy array representing the output of a render
         window, or ``vtkImageData``.
 
-    threshold : int, optional
+    threshold : int, default: 1
         Threshold tolerance for pixel differences.  This should be
         greater than 0, otherwise it will always return an error, even
         on identical images.
 
-    use_vtk : bool, optional
+    use_vtk : bool, default: True
         When disabled, computes the mean pixel error over the entire
         image using numpy.  The difference between pixel is calculated
         for each RGB channel, summed, and then divided by the number
@@ -116,11 +141,11 @@ def compare_images(im1, im2, threshold=1, use_vtk=True):
             if img._first_time:  # must be rendered first else segfault
                 img._on_first_render_request()
                 img.render()
-            if not hasattr(img, 'ren_win'):
+            if img.render_window is None:
                 raise RuntimeError(
                     'Unable to extract image from Plotter as it has already been closed.'
                 )
-            return image_from_window(img.ren_win, True, ignore_alpha=True)
+            return image_from_window(img.render_window, True, ignore_alpha=True)
         else:
             raise TypeError(
                 f'Unsupported data type {type(img)}.  Should be '
